{% theme "form" with header_title="Your Password" secondary_title="Change" %}
{# extends "theme/form-page.html" #}


{% block header_title %}Your Password{% endblock header_title %}
{% block secondary_title %}Change{% endblock secondary_title %}