{% extends "django_registration/base_registration.html" %}


{% block title %}Account activation error{% endblock %}
{% block body_class %}{{ block.super }} registration_activate{% endblock %}


{% block content_title %}
  <h2>Account activation error</h2>
{% endblock %}


{% block content %}
  <p>The activation key "{{ activation_key }}" is invalid and cannot be used to activate this account.</p>
{% endblock %}
