{% extends "trim/crud/base.html" %}
{% load static %}
{% block page_content %}

<form method="post">
    <ul class="crud-form">
        {{ form.as_ul }}
    </ul>
    <input type="submit" value="Confirm">
    {% csrf_token %}
</form>

{% endblock page_content %}

{% block footer_js %}
    <link rel="stylesheet" href="{% static 'trim/css/crud-form.css' %}">
{% endblock footer_js %}
