{% extends "base.html" %}

{% block content %}
    <div class="crud-page-content">
        {% block page_content %}
        <p>Default CRUD page content</p>
        {% endblock page_content %}
    </div>
{% endblock content %}
