

{% extends "account/base.html" %}

{% block hero_content %}
    {% include "v4/secondary-header.html" with primary="Login" secondary="Reset Sent" %}
{% endblock hero_content %}

{% block content %}
    {{ block.super }}

<div class="max-width-1200px">

<h3>Password reset sent</h3>
<p>We've emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.</p>

<p>If you don’t receive an email, please make sure you’ve entered the address you registered with, and check your spam folder.</p>
</div>

<div class="flex-center gap-1">
    <a href='{% url "account:login" %}' class="btn">Login</a>
    {% include "account/sub/register_button.html" %}
</div>

{% endblock %}
