{% extends "base.html" %}
{% load static wagtailcore_tags wagtailuserbar %}

{% block extra_css %}
    <link rel="stylesheet" type="text/css" href="{% static 'css/mock1.css' %}">

    <link rel="preconnect" href="https://fonts.googleapis.com">
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
    <link href="https://fonts.googleapis.com/css2?family=Roboto:ital,wght@0,100;0,300;0,400;0,500;0,700;0,900;1,100;1,300;1,400;1,500;1,700;1,900&family=Rubik:wght@439&display=swap" rel="stylesheet">
    <link href="https://fonts.googleapis.com/css2?family=Hind+Siliguri:wght@300;400;500;600;700&family=Hind+Vadodara:wght@300;400;500;600;700&display=swap" rel="stylesheet">
{% endblock %}

{% block content %}

{% if user.is_anonymous %}
    {% include "account/login_prompt.html" %}
{% else %}

    <h2>profile {{ user }}</h2>

    <ul>
        <li>name, email address</li>
        <li>connected logins</li>
        <li>addresses</li>
        <li>organisations</li>
        <li>puchases</li>
        <li>wishlists</li>
    </ul>
{% endif %}

    <p>This is your account area, find all your personal assets here.</p>
{% endblock %}
