{% extends "account/base.html" %}


{% block title %}Registration complete{% endblock %}
{% block body_class %}{{ block.super }} registration_complete{% endblock %}

{% block hero_title %}Registration complete{% endblock hero_title %}

{% block hero_header %}
    {% include "v1/secondary-header.html" with primary='Account' secondary='Registration complete' %}
{% endblock hero_header %}

{% block deep_content %}

    <p>We sent you an email with an activation link. Please activate your account with the link given within the email. If you choose not to activate your account it will eventually be deleted.</p>

    <p>Please check your email.</p>
{% endblock %}
