{% extends "base.html" %}

{% load static wagtailcore_tags wagtailuserbar %}

{% block content %}
    {% block inner_content %}
        {% block deep_content %}{% endblock %}
    {% endblock %}
{% endblock %}
