{% extends "django_registration/base_registration.html" %}


{% block title %}Log in{% endblock %}
{% block body_class %}{{ block.super }} login_form{% endblock %}


{% block content_title %}
  <h2>Log in</h2>
{% endblock %}


{% block content %}
  <form action="." method="post" class="login_form">{% csrf_token %}
    {{ form.as_p }}
    <p class="submit"><button type="submit" name="submit" value="Login">Log in</button></p>
    {% if next %}<input type="hidden" name="next" value="{{ next }}">{% endif %}
  </form>
{% endblock %}
