# Generated by Django 4.0.2 on 2022-10-19 00:59

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("account", "0006_alter_accountemail_user"),
    ]

    operations = [
        migrations.AddField(
            model_name="accountemail",
            name="uuid_token",
            field=models.CharField(
                blank=True, default=uuid.uuid4, max_length=255, null=True
            ),
        ),
    ]
