{% extends "django_registration/base_registration.html" %}


{% block title %}Registration closed{% endblock %}
{% block body_class %}{{ block.super }} registration_closed{% endblock %}


{% block content_title %}
  <h2>Registration closed</h2>
{% endblock %}


{% block content %}
  <p>Please try <a href="{% url registration_register %}">registering again</a>.</p>
{% endblock %}
