{% extends "account/base.html" %}
{% load link %}

{% block inner_content %}
    <h1>Forgot Password</h1>
    {% include "account/sub/primary_login_form.html" %}
    {% link "account:login" "Login" %}

{% endblock %}
