<div class="flex-down gap-1">

    <div class="grid">
        {% include "crud/form.html" %}
    </div>

    <div class="actions btns flex-down">
        {% include "account/sub/register_button.html" %}
        {% include "account/sub/social_buttons.html" %}
    </div>
</div>
