<a href='{% url "social:begin" "google-oauth2" %}'>Google</a>
<a href='{% url "social:begin" "linkedin-oauth2" %}'>Linkedin</a>
        <!-- <a href='{% url "social:begin" "twitter" %}'>Twitter</a> -->
