{% load updated_params %}
<div class="pagination">
    <span class="step-links">
        {% if page_obj.has_previous %}
            <a href="?{% updated_params page=1 %}">&laquo; first</a>
            <a href="?{% updated_params page=page_obj.previous_page_number %}">previous</a>
        {% endif %}

        <span class="current">
            Page {{ page_obj.number }} of {{ page_obj.paginator.num_pages }}.
        </span>

        {% if page_obj.has_next %}
            <a href="?{% updated_params page=page_obj.next_page_number %}">next</a>
            <a href="?{% updated_params page=page_obj.paginator.num_pages %}">last &raquo;</a>
        {% endif %}
    </span>
</div>
