{% extends "account/base.html" %}

{% block hero_content %}
    Password Change
{% endblock hero_content %}

{% block inner_content %}>
    {% include "account/sub/primary_login_form.html" %}
{% endblock %}
