{% extends "account/base.html" %}

{% block hero_content %}
    Login
{% endblock hero_content %}

{% block inner_content %}
    <div class="flex-down">
        <div class="flex-center">
            {% include "account/sub/primary_login_form.html" %}
        </div>
    </div>
{% endblock %}
