{% extends "account/base.html" %}

{% block hero_content %}
    {% include "v4/secondary-header.html" with primary="Login" secondary="Forgot Password" %}
{% endblock hero_content %}

{% block deep_content %}
    {{ block.super }}
    <p>Please enter your email address. If it exists in our system, you'll recieve a link to reset your password.</p>
    <div class="flex-center">
        {% include "account/sub/login_form.html" %}
    </div>

{% endblock %}
