{% extends "theme/form-page.html" %}

{% block header_title %}Your Email{% endblock header_title %}
{% block secondary_title %}Edit{% endblock secondary_title %}

{% block right_content %}
    <p>Change your email address to the preferred for contact and receipts. A confirmation email is sent to the both email addresses before the change occurs. <br>Please ensure to check the old and new accounts before accessing the site. </p>
{% endblock right_content %}
