

{% extends "account/base.html" %}

{% block hero_content %}
    {% include "v4/secondary-header.html" with primary="Login" secondary="Reset Sent" %}
{% endblock hero_content %}

{% block content %}
    {{ block.super }}

<div class="max-width-1200px">

<h3>Password reset complete</h3>
<p>You may now use your new password to login.</p>
</div>

{% endblock %}
